clear;
clc;
close all;


%%  Load RGB-XRF data
syndata = load('syn_data_Square_S_1024.mat');


im_XRF = syndata.airforce_xrf_LR;       % The LR XRF input image
im_RGB_orig = syndata.airforce_img;     % The HR RGB input image
im_XRF_GT = syndata.airforce_xrf_HR;    % The HR XRF ground truth image

clear syndata;



%%  Spatial Down-Sample
im_l_XRF = im_XRF;

im_RGB = im_RGB_orig;


S = hyperSpatialDown(size(im_RGB,1), size(im_RGB,2) ,5);    % The downsample matrix

%   Degrade the HR RGB image to LR RGB image
RGB_signal = hyperConvert2d(im_RGB);
RGB_l_signal = RGB_signal * S;
im_l_RGB = hyperConvert3D(RGB_l_signal, size(im_l_XRF,1), size(im_l_XRF,2), size(im_RGB,3));

figure;
imshow(im_l_RGB);

%% Reshape the XRF/RGB image to matrix form, each column of the matrix is a pixel

XRF_l_signal = reshape(im_l_XRF, [], size(im_XRF, 3));
XRF_l_signal = XRF_l_signal';


RGB_l_signal = reshape(im_l_RGB, [], size(im_RGB, 3));
RGB_l_signal = RGB_l_signal';



%%  Initialization using Coupled dictionary learning

XRF_l_signal = normc(XRF_l_signal);
RGB_l_signal = normc(RGB_l_signal);

both_signal = [ XRF_l_signal; RGB_l_signal];
both_signal = normc(both_signal);

nrSamples = size(both_signal, 2);

M = 50;     % Number of atroms in the dictionary
%   Parameters for LASSO
dlparam.mode = 1;
dlparam.lambda = 0.001;
dlparam.K = M;
dlparam.iter = 100;
dlparam.batchsize = nrSamples;
dlparam.posAlpha = 1;
dlparam.posD = 1;

%   Dictionary Learning using SPAMS toolbox
Db = mexTrainDL(both_signal, dlparam);

D_XRF = Db(1 : size(XRF_l_signal,1), :);
D_RGB = Db(size(XRF_l_signal,1) + 1 : end, :);

D_XRF = normc(D_XRF);
D_RGB = normc(D_RGB);

%%  Parameters for the proposed fusion based SR
supParam.maxIter = 5;     %   Total number of iteration
supParam.epsilon = 0.0000001;
supParam.p = M;             %   The number of atoms in the dictionary
supParam.TVlambda = 10^(-2);%   TV regularizer
supParam.scale_diff = 5;    %   Upscale factor
supParam.height = size(im_RGB_orig, 1);
supParam.width = size(im_RGB_orig, 2);
% supParam.kernel_var = kernel_var;
% supParam.kernel_sup = kernel_sup;

param.mode = 1;
param.pos = 0;
param.lambda = 0.01;
supParam.param = param;

%%  Using the method in "Hyperspectral Super-Resolution by Coupled Spectral Unmixing"

% [D_RGB, D_XRF, A, AS, Ah, ASh, stat]= dqq_SupResPALM_nv_GD_TV_Dh_adv_Square_S_one_D_xrf(XRF_l_signal, RGB_signal, D_XRF, D_RGB, supParam);
[D_RGB, D_XRF, Dh_XRF, A, AS, Ah, ASh, stat]= dqq_SupResPALM_nv_GD_TV_Dh_adv_Square_S(XRF_l_signal, RGB_signal, D_XRF, D_XRF, D_RGB, supParam);


A   = full(A);
AS  = full(AS);
Ah  = full(Ah);
ASh = full(ASh);

sparse_code_3D = hyperConvert3d((A), size(im_RGB, 1), size(im_RGB, 2));
sparse_code_3Dh = hyperConvert3d((Ah), size(im_RGB, 1), size(im_RGB, 2));



%%  Reconstruct XRF volume

im_XRF_recon = zeros(size(im_RGB_orig, 1), size(im_RGB_orig, 2), size(im_XRF,3));
im_XRF_recon_v = zeros(size(im_RGB_orig, 1), size(im_RGB_orig, 2), size(im_XRF,3));
im_XRF_recon_nv = zeros(size(im_RGB_orig, 1), size(im_RGB_orig, 2), size(im_XRF,3));


for i = 1 : size(im_RGB_orig, 1)
    
    for j = 1 : size(im_RGB_orig, 2)
        
        spec_cur = normc(D_XRF * squeeze(sparse_code_3D(i, j, :)) + Dh_XRF * squeeze(sparse_code_3Dh(i, j, :)));
        spec_cur_v = D_XRF * squeeze(sparse_code_3D(i, j, :));
        spec_cur_nv = Dh_XRF * squeeze(sparse_code_3Dh(i, j, :));
        
        im_XRF_recon(i, j, :) = spec_cur;
        im_XRF_recon_v(i, j, :) = spec_cur_v;
        im_XRF_recon_nv(i, j, :) = spec_cur_nv;
        
    end
    
end


%%  Normalize im_XRF
im_XRF_norm = zeros(size(im_XRF, 1), size(im_XRF, 2), size(im_XRF,3));

for i = 1 : size(im_XRF, 1)
    
    for j = 1 : size(im_XRF, 2)
        
        im_XRF_norm(i, j, :) = normc( squeeze(im_XRF(i, j, :)));
        
    end
    
end



%% Save video for resonctructed patch volumn: 3 volumes
patch_index = 3;

% filename = ['Bedroom_reconstruction_xl_' num2str(patch_index) '_atom_' num2str(size(D_XRF,2)) '.avi'];
% filename = ['Bedroom_reconstruction_xl_adv_' num2str(patch_index) '_atom_GD_TV_DDh_GS_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_kv_' num2str(kernel_var) '_ks_' num2str(kernel_sup) '.avi'];
filename = ['syn_data_TV_RGB_XRF_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_square_s.avi'];

v = VideoWriter(filename,'Uncompressed AVI');
open(v);

% volumn_max = max(max(max(data_reconstruction_nonorm)));

im_XRF_recon(im_XRF_recon < 0) = 0;
im_XRF_norm(im_XRF_norm < 0) = 0;

for j = 1 : size(im_XRF_norm,3)
    
%     j
    
    img_tmp = zeros(size(im_XRF_recon, 1) + size(im_XRF_norm, 1), size(im_XRF_recon, 2) * 2);
    
    img_tmp(1 : size(im_XRF_recon, 1)                                               , 1 : size(im_XRF_recon, 2)) = im_XRF_recon(:,:,j);
    img_tmp(1 + size(im_XRF_recon, 1) : size(im_XRF_recon, 1) + size(im_XRF_norm, 1), 1 : size(im_XRF_norm, 2))  = im_XRF_norm(:,:,j);
    
    img_tmp = img_tmp / max(max(img_tmp));
        
    img_tmp = repmat(img_tmp, [1 1 3]);
    
    img_tmp(1 :  size(im_XRF_recon, 1), 1 + size(im_XRF_recon, 2) : 2 * size(im_XRF_recon, 2), :) = im_RGB;

    imshow(img_tmp,'border','tight');
    writeVideo(v,img_tmp);
   
end
close(v);

filename = ['syn_data_TV_RGB_XRF_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_square_s.mat'];
save(filename,'-v7.3');


%% Save video for resonctructed patch volumn: Result only
patch_index = 3;

% filename = ['Bedroom_reconstruction_xl_' num2str(patch_index) '_atom_' num2str(size(D_XRF,2)) '.avi'];
filename = ['syn_data_TV_RGB_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_square_s_result_only.avi'];

v = VideoWriter(filename,'Uncompressed AVI');
open(v);

% volumn_max = max(max(max(data_reconstruction_nonorm)));

im_XRF_recon(im_XRF_recon < 0) = 0;
im_XRF_norm(im_XRF_norm < 0) = 0;

for j = 1 : size(im_XRF_norm,3)
    
%     j
    
    
    img_tmp = im_XRF_recon(:,:,j);
    
    img_tmp = img_tmp / max(max(im_XRF_norm(:,:,j)));
    
    img_tmp(img_tmp > 1) = 1;
    img_tmp(img_tmp < 0) = 0;

    imshow(img_tmp,'border','tight');
%     pause(1/200);
    writeVideo(v,img_tmp);
   
end
close(v);

%% Save video for resonctructed patch volumn Visible part
patch_index = 3;

% filename = ['Bedroom_reconstruction_xl_' num2str(patch_index) '_atom_' num2str(size(D_XRF,2)) '.avi'];
filename = ['syn_data_TV_RGB_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_square_s_result_only_visible.avi'];

v = VideoWriter(filename,'Uncompressed AVI');
open(v);

% volumn_max = max(max(max(data_reconstruction_nonorm)));

im_XRF_recon_v(im_XRF_recon_v < 0) = 0;
im_XRF_norm(im_XRF_norm < 0) = 0;

for j = 1 : size(im_XRF_norm,3)
    
%     j
    
    
    img_tmp = im_XRF_recon_v(:,:,j);
    
    img_tmp = img_tmp / max(max(im_XRF_norm(:,:,j)));
    
    img_tmp(img_tmp > 1) = 1;
    img_tmp(img_tmp < 0) = 0;

    imshow(img_tmp,'border','tight');
%     pause(1/200);
    writeVideo(v,img_tmp);
   
end
close(v);

%% Save video for resonctructed patch volumn NonVisible Part
patch_index = 3;

% filename = ['Bedroom_reconstruction_xl_' num2str(patch_index) '_atom_' num2str(size(D_XRF,2)) '.avi'];
filename = ['syn_data_TV_RGB_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_square_s_result_only_non_visible.avi'];

v = VideoWriter(filename,'Uncompressed AVI');
open(v);

% volumn_max = max(max(max(data_reconstruction_nonorm)));

im_XRF_recon_nv(im_XRF_recon_nv < 0) = 0;
im_XRF_norm(im_XRF_norm < 0) = 0;

for j = 1 : size(im_XRF_norm,3)
    
%     j
    
    
    img_tmp = im_XRF_recon_nv(:,:,j);
    
    img_tmp = img_tmp / max(max(im_XRF_norm(:,:,j)));
    
    img_tmp(img_tmp > 1) = 1;
    img_tmp(img_tmp < 0) = 0;

    imshow(img_tmp,'border','tight');
%     pause(1/200);
    writeVideo(v,img_tmp);
   
end
close(v);



%%  Normalize im_XRF_GT
im_XRF_GT_norm = zeros(size(im_XRF_GT, 1), size(im_XRF_GT, 2), size(im_XRF_GT,3));

for i = 1 : size(im_XRF_GT, 1)
    
    for j = 1 : size(im_XRF_GT, 2)
        
        im_XRF_GT_norm(i, j, :) = normc( squeeze(im_XRF_GT(i, j, :)));
        
    end
    
end

%%  Calculate the SR result's Error
SR_Error = norm(im_XRF_recon(:) - im_XRF_GT_norm(:), 2)

filename = ['syn_data_TV_RGB_M_' num2str(patch_index) '_atom_GD_TV_DDh_GS_M_' num2str(size(D_XRF,2)) '_TVlambda_' num2str(supParam.TVlambda) '_square_S.mat'];
save(filename,'-v7.3');
